// 
// ComboBoxEntryBackend.cs
//  
// Author:
//       Lluis Sanchez <lluis@xamarin.com>
// 
// Copyright (c) 2011 Xamarin Inc
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
using System;
using Xwt.Backends;

namespace Xwt.GtkBackend
{
	public class ComboBoxEntryBackend: ComboBoxBackend, IComboBoxEntryBackend
	{
		TextEntryBackend entryBackend;
		
		protected override Gtk.Widget CreateWidget ()
		{
			var c = new Gtk.ComboBoxEntry ();
			c.Clear ();
			entryBackend = new CustomComboEntryBackend ((Gtk.Entry)c.Child);
			return c;
		}
		
		public override void Initialize ()
		{
			base.Initialize ();
			((Gtk.ComboBoxEntry)Widget).TextColumn = 0;
		}
		
		public void SetTextColumn (int column)
		{
			((Gtk.ComboBoxEntry)Widget).TextColumn = column;
		}
		
		public ITextEntryBackend TextEntryBackend {
			get {
				return entryBackend;
			}
		}
	}
	
	class CustomComboEntryBackend: TextEntryBackend
	{
		Gtk.Entry entry;
		
		public CustomComboEntryBackend (Gtk.Entry entry)
		{
			this.entry = entry;
		}
		
		public override void Initialize ()
		{
			Widget = entry;
			entry.Show ();
		}
	}
}

